#define FM_AS65_MESSUNG
#define REG_LAUFZEIT_TEST

//#define CORE1_AKTIV  // experimentell 
#define INT128
/*
mino Pico-FMeter-AS6501

***************** GB start *********************
For a reciprocal frequency measurement, the number of incoming events are counted 
and synchronized with the exact times. 

Timing: 
A 74AUP1G74 DFF is used to synchronize the times. After setting the D-input 
to '1' its Q-output ('CAP1') follows to '1' at the next pos. edge of Fin clock. 
Inside the TDC AS6501 'CAP1' captures the reference counter, which runs at 10 MHz, 
and another stop counter. The stop counter finely resolves 
the 100 ns of the 10 MHz clock to 0x10000 intermediate levels. 
After that the 'INTN' output is set to '0' and the DFF is thus blocked. 
The counters can be read out gradually. 

Event measurement: 
The input pulses at 'FIN' are counted by the 16-bit PWM_CH2 counter using
PWM2B input for counting pos. edges.

Time stamp: 
Time measurement and event counting still have to be merged. The exact 
synchronous time is requested by the DFF with 'CAP1' as described.
'CAP1' also triggers DMA_CH0 channel: The current count of PWM_CH2_CTR is 
transfered to PWM_CH2_event variable.. 
This 16 bit counter is sufficient, as it is read about 50,000 times 
at high input frequencies. At low frequencies it is read with each new input pulse.
 
PWM_CH2_event contains the synchronous events to the time event counter of AS6501. 
The eff.  number of events between two readings is as follows: 
(new_state - old_state) & 0xffff 
and for TDC values with their 40 bits as: 
(new_state - old_state) & 0xffffffffff. 
Both values (PWM_CH2_event and TDC-count) are used as a time stamp for further 
evaluation. 

To avoid PIO0_SM0 stands still (blocked if FIFO is full) after each 
PWM_CH2_CTR-transfer, DMA_CH0 triggers channel DMA_CH1, which empties 
PIO0_SM0 FIFO and then activates DMA_CH0 again. 
This action runs without ISR in the hardware. 


Regression calculation: 
To increase the measurement resolution, the recorded time stamps are processed 
until the end of the specified measurement interval frequency calculation takes 
place. In order not to overload the controller, a maximum of 50000 time stamps/s 
(@ SysCoreClock = 133 MHz) are evaluated. 
With PWM-timer 6, the 'INT_REQ' input is queried every 20 s. If it is set to 
'0' a 'CAP1' event was previously triggered and new values are ready to be read. 
'handle_F1_reg()' performs the preprocessing where 'DMA_CH1_event' must be read 
first! 

By reading out the AS6501, a new time stamp is automatically requested: 
'INTN' is set to '1' again and therefore D-input too.

End of measurement interval: 
After the specified measurement interval (F1_messzeit) has expired, 
the routine 'F1_messung()' performes the final frequency calculation: 
'bewerte_F1_messung() '.

In addition, it is checked which values are to be output in which form and 
whether the ext. 4:1 prescaler must be switched on or off.
 
The rest of the features should be apparent from the comments.

***************** GB end *********************

reziproke Frequenzmessung mit RP2040 und TDC AS6501 fuer hohe Aufloesung

Fuer eine reziproke Frequenzmessung werden die Anzahl der eintreffenden Ereignisse
gezaehlt und synchron dazu die genauen Zeitpunkte.

Zeitmessung:
Zur Synchronisierung der Zeitpunkte dient ein 74AUP1G74 DFF, welches nach Setzen
des D-Eingangs beim naechsten Takt seinen Q-Ausgang auf '1' setzt. Diese 0->1 
Flanke (Signal 'CAP1') speichert im TDC AS6501 die internen Zaehlerstaende eines 
Referenzzaehlers, der mit 10 MHz laeuft, und eines weiteren Stop-Zaehlers, der 
die 100 ns des 10 MHz Taktes auf 0x10000 Zwischenstufen fein aufloest.
Anschlieend wird der Ausgang 'INTN' auf '0' gesetzt und das DFF damit gesperrt.
Die Zaehler koennen anschlieissend ausgelesen werden. 
Soweit die Zeitmessung.

Ereignismessung:
Die Eingangsimpulse an 'FIN' werden per 16 Bit PWM-Zaehler PWM_CH2_CTR gezaehlt.
Der PWM2B-Eingang zaehlt die pos. Flanken von 'Fin'.

Zeitstempel:
Zeitmessung und Ereigniszaehlung muessen noch zusammengefuehrt werden. Der genaue, 
synchrone Zeitpunkt wird wie beschrieben durch das DFF mit 'CAP1' angefordert. 

Per 'CAP1'-Signal vom DFF wird zudem der aktuelle Zaehlerstand von PWM_CH2_CTR in 
die Variable PWM_CH2_event per DMA_CH0 uebertragen. Ein 16 Bit Zaehler ist 
ausreichend, da er bei hoher Eingangsfrequenz ca 50000 Mal ausgelesen wird. 
Bei niedrigen Frequenzen sogar bei jedem neuen Eingangsimpuls.
In PWM_CH2_event steht der zum AS6501-Zeitpunkt synchrone Zaehlerstand des 
Ereigniszaehlers.
 
Die eff. Anzahl der Ereignisse zwischen zwei Auslesungen ergibt sich als:
(neuer_Zustand - alter_Zustand) & 0xffff 
und fuer die Zeitmessung mit ihren 40 Bit als: 
(neuer_Zustand - alter_Zustand) & 0xffffffffff.

Beide Werte werden als Zeitstempel fuer die weitere Auswertung verwendet.
Damit PIO0_SM0 nicht stehen bleibt (blockiert durch ein volles FIFO), tiggert 
DMA_CH0 den Kanal DMA_CH1, der das PIO0_SM0 FIFO leert und danch DMA_CH0 wieder 
aktiviert. 
Diese Aktion laueft ohne ISR in der Hardware ab.


Regressionsberechnung:
Zur Erhoehung der Messaufloesung werden die erfassten Zeitstempel solange
aufbereitet, bis nach Ablauf des vorgegeben Messintervalls die abschliessende
Frequenzberechnung stattfindet.
Um den Controller nicht zu ueberlasten, werden maximal 50000 Zeitstempel/s ausgewertet.
Mit Timer 6 wird alle 20 s der Eingang 'INT_REQ' abgefragt. Liegt er auf '0'
wurde zuvor ein 'CAP1'-Ereignis ausgeloest und neue Werte sind zum Auslesen bereit.
'handle_F1_reg()' fhrt die Vorverarbeitung durch, wobei 'DMA_CH1_event'
zuerst gelesen werden muss!
Duch Auslesen des AS6501 wird automatisch ein neuer Zeitstempel angefordert:
Mit 'INTN' -> '1' wird auch D-Eingang wieder auf '1' gesetzt.

Ende des Messintervalls:
Nach Ablauf des vorgegebenen Messintervalls (DEF_F1_MESSZEIT) wird in der 
Routine 'F1_messung()' die abschlieende Frequenzberechnung durchgefhrt: 
'bewerte_F1_messung()'
Nebenbei wird noch geprft, welche Werte in welcher Form ausgegeben werden sollen
und ob eventuell der ext. 4:1 Vorteiler zugeschaltet werden muss.

Die uebrigen Funktionen sollten aus den Kommentaren erkennbar sein.


2023-04-30
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#include "System-Pico-FM-AS65.h"
#include <dma_req.h>
#include <intrinsics.h>

extern void     init_AS6501(void);
extern uint64_t lese_AS6501(void);

#ifdef CORE1_AKTIV
#define STACK_LEN   200   // je 32 bit
enum core1_bef{CORE1_STOP=0, CORE1_RUN, CORE1_ACK};

volatile uint8_t core1_cmd;
uint32_t core1_stack[STACK_LEN];
void starte_core1(void);
#endif


#define REG_ABTASTRATE    50000

#define PWM_COUNT         0x10000       // 16 Bit Zaehler per PWM-Kanal
#define PWM_COUNT_MASK    (PWM_COUNT-1)
#define AS65_MASK         0xffffffffff  // Zeitmessung 40 Bit breit


#define F1H_VORTEILER     4             // gesamter Faktor fuer Skalierung
// Umschaltschwellen fuer Vorteiler vorgeben >= 15 MHz ein, <= 11 MHz aus
// wegen 65535/ms: maximal 65 MHz moeglich
#define F1V_FMIN  (11000/F1H_VORTEILER) // wegen 1 ms Intervall in kHz
#define F1V_FMAX   15000                // dto.

#define DEF_FAKTOR        1
#define DEF_UPM_DIVISOR   1


enum {SM0 = 0, SM1, SM2, SM3} SMx_KANAL;
enum {CH0 = 0, CH1, CH2, CH3} DMA_CHx_Kanal;

#define TEMP_STR_LEN      50          // fuer Ergebnis String
#define MAX_ERGEBNIS_STR  20          // Ausgabelaenge begrenzen

double rundung[] = {5e0, 5e-1,5e-2,5e-3,5e-4,5e-5,
                    5e-6,5e-7,5e-8,5e-9,5e-10,5e-11,
                    5e-12,5e-13,5e-14,5e-15};

char f_dim[][10][6]= {{" GHz"," MHz"," kHz"," Hz "," mHz"," sek"," ms "," us "," ns "," ps "},
                      {"E+9","E+6","E+3","E+0","E-3","E+0","E-3","E-6","E-9","E-12"}};  // Dimensionen zum Messwert
char u_dim[][5][6]=  {{" Grpm"," Mrpm"," krpm"," rpm"," mrpm"},
                      {"E+9","E+6","E+3","E+0","E-3"}};

enum anzeige {WANDEL_F=0, WANDEL_P, WANDEL_D};

uint32_t  F1_time_stamp,        // Zeitpunkt von PIO0-SM0
          pio0_sm1_dummy,       // Ereignis von PIO0-SM1
          PWM_CH2_event,        // abwaerts zaehlend: Summe Ereignisse zu CAP1
          dma_dummy;

uint16_t  led_count,
          led_ein_zeit = TYP_LED_ZEIT;    // Blinkzeit

uint8_t   ausgabe_sperre = 1,             // keine LCD Datenausgage
          anzeige_funktion = F_UND_P,     // F+P, nur F, nur P, nur UPM oder auch aus -1
          update_LCD,                     // LCD neu beschriften
          nur_MHz,                        // festes Format x.xxx MHz
          auto_vorteiler_umschaltung=1,   // 1 = mit Neustart, 0 = schnell
          F1_reg_modus = OPTIMIERT,
          anzeige_format = 0,             // bit0: Dimension oder Exponent; bit1: . oder , 
          ser_ausgabe_kanal = F1;         // ser. Datenausgabe: F, P, UPM oder auch aus -1


double    F1_ergebnis;                    // in Hz
double    eff_zeit1;                      // in s fuer automatische Stellenanzahl
double    F1_taktfrequenz;                // Platzhalter, nicht verwendet
double    F1_takt = (double)AS6501_REF * REFCLK_AUFLOESUNG;;
uint32_t  F1_ms_grob;                     // grobe Frequenzmessung mit 1 ms Intervall
uint8_t   auto_vorteiler_aktiv,           // Signaleingang: 0=ohne, 1= mit Vorteiler
          F1_vorteiler_aktiv;             // manuelle Zuschaltung, wenn extern vorhanden

uint32_t  F1_messzeit = DEF_F1_MESSZEIT,  // Sollintervall
          F1_timeout = DEF_TIMEOUT,       // Abbruch bei fehlendem Signal
          F1_stellen = DEF_STELLEN,       // Vorgabewert; 0 = automatisch
          F1_faktor  = DEF_FAKTOR,        // 1
          F1_upm_divisor = DEF_UPM_DIVISOR;  // ebenso
                    
volatile uint8_t
          F1_messwert_vorhanden;          // gueltige Messung
                                   
volatile uint32_t
          F1_mess_dauer,                  // in ms
          F1_timeout_counter;             // in ms
uint64_t  F1_start_zeit,                  // rel. Zeit: Beginn der Messung
          F1_end_zeit,                    // Zeitpunkt der Auswertung
          F1_mess_zeit;                   // genaue Zeit der Messung
                    
uint32_t  F1_mess_ereignisse,             // Impulse der Messung
          F1_start_ereignis,              // Impulse zu Beginn der Messung
          F1_end_ereignis;                // Impulse am Ende der Messung

int32_t   F1_offset = 0,                  // aktuell verwendeter Wert
          Fintern_offset,                 // fuer lokalen TCXO
          Fextern_offset;                 // fuer ext. 10 MHz

void      F1_neustart(void);              // verwerfen der laufenden Messung + Neustart



// Variablen fuer Regressionsberechnung
uint32_t  reg_n, temp_reg_n;              // Anzahl Ereignisse bei Regression
volatile  uint32_t reg_ereignisse;        // und Endwert zur weiteren Auswertung
double    reg_xi, reg_yi,                 // summe fortlaufender Einzelwerte  
          temp_reg_xi, temp_reg_yi;                      
double    reg_sum_xi, reg_sum_yi, 
          temp_reg_sum_xi, temp_reg_sum_yi; // Summe aller Intervalle

double    reg_sum_xiyi, temp_reg_sum_xiyi,  // Summe xi * yi
          reg_sum_xi2, temp_reg_sum_xi2;    // Summe xi


// Variablen fr Regressionsberechnung 64
typedef struct {
  uint64_t  reg_n_64;                          // Anzahl Ereignisse bei Regression
  volatile  uint64_t reg_ereignisse_64;        // und Endwert
  uint64_t  reg_xi_64,                         // Summe aller Intervalle
            reg_yi_64, 
            reg_yi_end_64,         // lesbare Endwerte 
            reg_sum_xi_64, reg_sum_yi_64,
            zeitpunkt_64;

  uint64_t  xixi_sum_128_h, xixi_sum_128_l;
  uint64_t  xiyi_sum_128_h, xiyi_sum_128_l;
} REG_STRUCT;

static REG_STRUCT  F1_reg, Fx_reg;

void copy_regression_64(REG_STRUCT *Fdest_reg, REG_STRUCT *Fsrc_reg)
{
  Fdest_reg->reg_n_64 = Fsrc_reg->reg_n_64;                    // alle Werte der Regressionsberechnung
  Fdest_reg->reg_xi_64 = Fsrc_reg->reg_xi_64;                  //  umkopieren
  Fdest_reg->reg_yi_64 = Fsrc_reg->reg_yi_64;  
  Fdest_reg->reg_sum_xi_64 = Fsrc_reg->reg_sum_xi_64;
  Fdest_reg->reg_sum_yi_64 = Fsrc_reg->reg_sum_yi_64;
  Fdest_reg->xixi_sum_128_h = Fsrc_reg->xixi_sum_128_h;
  Fdest_reg->xixi_sum_128_l = Fsrc_reg->xixi_sum_128_l;
  Fdest_reg->xiyi_sum_128_h = Fsrc_reg->xiyi_sum_128_h;
  Fdest_reg->xiyi_sum_128_l = Fsrc_reg->xiyi_sum_128_l;
}

void loesche_regression_64(REG_STRUCT *Fx_reg)
{
  Fx_reg->reg_n_64 = 0;                    // alle Werte der Regressionsberechnung
  Fx_reg->reg_xi_64 =  0;                  // lschen
  Fx_reg->reg_yi_64 = 0;  
  Fx_reg->reg_sum_xi_64 = 0;
  Fx_reg->reg_sum_yi_64 = 0;
  Fx_reg->xixi_sum_128_h = 0;
  Fx_reg->xixi_sum_128_l = 0;
  Fx_reg->xiyi_sum_128_h = 0;
  Fx_reg->xiyi_sum_128_l = 0;
}

void mul64to128(uint64_t faktor1, uint64_t faktor2, uint64_t *prod_high, uint64_t *prod_low)
{
  uint64_t temp1, temp2, temp3;
  uint64_t faktor_11 = (faktor1 & 0xffffffff);
  uint64_t faktor_21 = (faktor2 & 0xffffffff);
  uint64_t prod_temp = (faktor_11 * faktor_21);
  
  temp1 = (prod_temp & 0xffffffff);
  temp2 = (prod_temp >> 32);
    
  faktor1 >>= 32;
  prod_temp = (faktor1 * faktor_21) + temp2;
  temp2 = (prod_temp & 0xffffffff);
  temp3 = (prod_temp >> 32);
    
  faktor2 >>= 32;
  prod_temp = (faktor_11 * faktor2) + temp2;
  temp2 = (prod_temp >> 32);
    
  *prod_high = (faktor1 * faktor2) + temp3 + temp2;
  *prod_low = (prod_temp << 32) + temp1;
}

void add128to128(uint64_t sum_high, uint64_t sum_low, uint64_t *res_high, uint64_t *res_low)
{
uint64_t temp = *res_low;
  *res_low += sum_low;
  if(*res_low < temp) (*res_high)++;    // Ueberlauf aufgetreten
  *res_high += sum_high;
}  

double from128to64double(uint64_t high, uint64_t low)
{
uint64_t temp_l = low, temp_h = high;
uint64_t faktor = 1;
double res;
  while(temp_h) {         // solange obere 64 Bit != 0
    temp_l += 1;          // Rundung
    temp_l >>= 1;
    faktor <<= 1;
    if(temp_h & 1) temp_l |= 0x8000000000000000;  // MSB ergnzen
    temp_h >>= 1;
  }  
  res = (double) temp_l * (double) faktor;
  return(res);
}  

void handle_F1_reg_64(void)
{
static uint64_t temp_low, temp_high; 
static uint8_t akt_vorteiler=255, discard_next;  // zum Umschalten des Vorteilers
// Test auf Vorteiler Umschaltung
  if(akt_vorteiler != auto_vorteiler_aktiv) {
    akt_vorteiler = auto_vorteiler_aktiv;
    if(akt_vorteiler) {
      SIO->GPIO_OUT_SET = BIT(PRE_MUX_OUT);
    } else {
      SIO->GPIO_OUT_CLR = BIT(PRE_MUX_OUT);
    }
    discard_next = 3;                   // drei nachfolgende Ergebisse verwerfen
    if(auto_vorteiler_umschaltung) {    // 1 = Neustart der laufenden Messung erzwingen
      F1_neustart();                    // laufende Messung sofort abschliessen
    }  
  }
#ifdef REG_LAUFZEIT_TEST
  SIO->GPIO_OUT_SET = BIT(GPIO18)  ;
#endif

// Zeitstempel fuer Einzelintervall lesen und auswerten
// Ereignisse immer zuerst!
  F1_end_ereignis = PWM_CH2_event;      // von PWM->CH2_CTR
  F1_mess_ereignisse = (F1_end_ereignis - F1_start_ereignis) & PWM_COUNT_MASK;
  F1_start_ereignis = F1_end_ereignis;
  
  F1_end_zeit = lese_AS6501();
  F1_mess_zeit = (F1_end_zeit-F1_start_zeit) & AS65_MASK;
  F1_start_zeit = F1_end_zeit;

  
  if(F1_mess_ereignisse) {              // nur gueltige Messwerte verwenden
    if(discard_next) {                  // Werte nach dem Umschalten verwerfen
      discard_next--;
    } else {
// Anzahl und Einzelwerte fuer die Regression verarbeiten 
      if(akt_vorteiler) 
        F1_mess_ereignisse *= F1H_VORTEILER;   // skalieren
      
      F1_reg.reg_n_64++;
      F1_reg.reg_yi_64 += (uint64_t)F1_mess_ereignisse;
      F1_reg.reg_xi_64 += F1_mess_zeit;
      F1_reg.reg_sum_xi_64 += F1_reg.reg_xi_64;
      F1_reg.reg_sum_yi_64 += F1_reg.reg_yi_64;
      mul64to128(F1_reg.reg_xi_64, F1_reg.reg_yi_64, &temp_high, &temp_low);
      add128to128(temp_high, temp_low, &F1_reg.xiyi_sum_128_h, &F1_reg.xiyi_sum_128_l);
      mul64to128(F1_reg.reg_xi_64, F1_reg.reg_xi_64, &temp_high, &temp_low);
      add128to128(temp_high, temp_low, &F1_reg.xixi_sum_128_h, &F1_reg.xixi_sum_128_l); 
    }  
  } 
#ifdef REG_LAUFZEIT_TEST
  SIO->GPIO_OUT_CLR = BIT(GPIO18)  ;
#endif
}  

// Auswertung nach Ablauf der Messdauer starten  
double bewerte_F1_messung_64(void) 
{
static double reg_divisor;
double F1_temp;
double d_temp1, d_temp2;
double reg_xq_64, reg_yq_64;              // Mittelwerte xi/n und yi/n

#ifdef CORE1_AKTIV
  core1_cmd = CORE1_STOP;
  while(core1_cmd != CORE1_ACK);
#else  
  PWM->CH6_CSR =  0;                      // disable reg-timer
#endif

  copy_regression_64(&Fx_reg, &F1_reg);
  loesche_regression_64(&F1_reg);
#ifdef CORE1_AKTIV
  core1_cmd = CORE1_RUN;
#else  
  PWM->CH6_CSR =  1;                      // und neu erfassen
#endif
      d_temp1 = from128to64double(Fx_reg.xiyi_sum_128_h, Fx_reg.xiyi_sum_128_l);
      d_temp2 = from128to64double(Fx_reg.xixi_sum_128_h, Fx_reg.xixi_sum_128_l);
      reg_xq_64 = (double)Fx_reg.reg_sum_xi_64/(double)Fx_reg.reg_n_64;
      reg_yq_64 = (double)Fx_reg.reg_sum_yi_64/(double)Fx_reg.reg_n_64;
      reg_divisor = (d_temp2 - (double)Fx_reg.reg_n_64 * (reg_xq_64 * reg_xq_64));
      if((F1_reg_modus != NORMAL) && Fx_reg.reg_n_64 >= 2 && reg_divisor != 0.0) {   // Regression erst ab zwei Werten          
        F1_temp = (d_temp1 - (double)Fx_reg.reg_n_64 * (reg_xq_64 * reg_yq_64)) / reg_divisor;
      } else {
       F1_temp = ((double)Fx_reg.reg_yi_64/(double)Fx_reg.reg_xi_64);
      }
      F1_temp *= F1_takt;                   // skalieren
// Endwerte fr Debugger  
      reg_ereignisse = Fx_reg.reg_n_64;
      eff_zeit1 = Fx_reg.reg_xi_64/F1_takt; // Gesamtzeit fr autom. Stellen
  return(F1_temp);
}





// F1-offset fuer lokalen/externen Takt
void setze_F1_offset()
{
  if(ex_ref_aktiv == 0) {
    F1_offset = Fintern_offset;
  } else {
    F1_offset = Fextern_offset;
  } 
}

void loesche_regression(void)
{
  reg_n = 0;                          // alle Werte der Regressionsberechnung
  reg_xi = reg_yi = 0;                // loeschen
  reg_sum_xi = reg_sum_yi = 0;
  reg_sum_xiyi = reg_sum_xi2 = 0;
}

// ISR zu PWM6 mit 50 kHz
// jedes (Teil-)Intervall berechnen und weiterverarbeiten
void handle_F1_reg(void)
{
static uint8_t akt_vorteiler=255, discard_next;  // zum Umschalten des Vorteilers
// Test auf Vorteiler Umschaltung
  if(akt_vorteiler != auto_vorteiler_aktiv) {
    akt_vorteiler = auto_vorteiler_aktiv;
    if(akt_vorteiler) {
      SIO->GPIO_OUT_SET = BIT(PRE_MUX_OUT);
    } else {
      SIO->GPIO_OUT_CLR = BIT(PRE_MUX_OUT);
    }
    discard_next = 3;                   // drei nachfolgende Ergebisse verwerfen
    if(auto_vorteiler_umschaltung) {    // 1 = Neustart der laufenden Messung erzwingen
      F1_neustart();                    // laufende Messung sofort abschliessen
    }  
  }
#ifdef REG_LAUFZEIT_TEST
  SIO->GPIO_OUT_SET = BIT(GPIO18)  ;
#endif
// Zeitstempel fuer Einzelintervall lesen und auswerten
// Ereignisse immer zuerst!
  F1_end_ereignis = PWM_CH2_event;      // von PWM->CH2_CTR
  F1_mess_ereignisse = (F1_end_ereignis - F1_start_ereignis) & PWM_COUNT_MASK;
  F1_start_ereignis = F1_end_ereignis;
  
  F1_end_zeit = lese_AS6501();
  F1_mess_zeit = (F1_end_zeit-F1_start_zeit) & AS65_MASK;
  F1_start_zeit = F1_end_zeit;
  
  if(F1_mess_ereignisse) {              // nur gueltige Messwerte verwenden
    if(discard_next) {                  // Werte nach dem Umschalten verwerfen
      discard_next--;
    } else {
// Anzahl und Einzelwerte fuer die Regression verarbeiten 
      reg_n++;                          // anzahl der Einzelwerte
      if(akt_vorteiler)                 // wenn aktiv
        F1_mess_ereignisse *= F1H_VORTEILER;   // skalieren
      reg_xi += F1_mess_ereignisse;     // Einzelwerte addieren
      reg_yi += F1_mess_zeit;           // dto
      reg_sum_xi += reg_xi;             // Summe aus Summe der Einzelwerte
      reg_sum_yi += reg_yi;             // dto
      reg_sum_xiyi += reg_xi*reg_yi;    // Summe der Produkte
      reg_sum_xi2  += reg_xi*reg_xi;    // Summe der Quadrate
     }  
  }  
#ifdef REG_LAUFZEIT_TEST
  SIO->GPIO_OUT_CLR = BIT(GPIO18)  ;
#endif  
}  

// Auswertung nach Ablauf der Messdauer starten  
double bewerte_F1_messung(void) 
{
double temp_reg_divisor;
double temp_reg_xq;
double temp_reg_yq;
double F1_temp;

#ifdef CORE1_AKTIV
  core1_cmd = CORE1_STOP;
  while(core1_cmd != CORE1_ACK);
#else  
  PWM->CH6_CSR =  0;                      // disable reg-timer
#endif  
  uint32_t temp_reg_n = reg_n;            // alle Werte der Regressionsberechnung
  temp_reg_xi = reg_xi;                   // umkopieren
  temp_reg_yi = reg_yi;                
  temp_reg_sum_xi = reg_sum_xi;
  temp_reg_sum_yi = reg_sum_yi;
  temp_reg_sum_xiyi = reg_sum_xiyi;
  temp_reg_sum_xi2 = reg_sum_xi2;
  loesche_regression();                   // loeschen
#ifdef CORE1_AKTIV
  core1_cmd = CORE1_RUN;
#else  
  PWM->CH6_CSR =  1;                      // und neu erfassen
#endif
  
  temp_reg_xq = temp_reg_sum_xi / temp_reg_n;
  temp_reg_yq = temp_reg_sum_yi / temp_reg_n;
  temp_reg_divisor = (temp_reg_sum_xi2 - temp_reg_n * (temp_reg_xq * temp_reg_xq));
  if((F1_reg_modus != NORMAL) && temp_reg_n >= 2 && temp_reg_divisor != 0.0) {  // Regression erst ab zwei Werten          
    F1_temp = F1_takt / ((temp_reg_sum_xiyi - temp_reg_n * (temp_reg_xq * temp_reg_yq)) / temp_reg_divisor);
  } else {                                          // wenn nur ein Ereignis in temp_reg_xi
    F1_temp = ((double)temp_reg_xi/(double)temp_reg_yi)*F1_takt;  // Berechnung ohne Regression
  }  
// Endwerte merken      
  reg_ereignisse = temp_reg_n;            // fuer Berechnung der Stellenanzahl           
  eff_zeit1 = temp_reg_yi / F1_takt;      // dto. eff. Messzeit in s
  return(F1_temp);
}

uint8_t berechne_stellen(double zeit, uint32_t ereignisse)
{
static double offset;
uint8_t stellen = MIN_STELLEN;	          // minimal
  if(ereignisse > 50) zeit *= 10;         // bei Regression mehr Stellen zulassen
  if(ereignisse > 5000) zeit *= 10;       // eine weitere
  if(ereignisse > 500000) zeit *= 10;     // noch eine weitere
  if(zeit+offset >= 990.0) {
    stellen = 13;
    offset = 100.0;
  } else if(zeit+offset >= 99.0) {
    stellen = 12;
    offset = 10.0;
  } else if(zeit+offset >= 9.9) {         // Anzahl der gltigen Stellen
    stellen = 11;                         // verringert sich bei krzeren Messzeiten
    offset = 1.0;
  } else if(zeit+offset >= 9.9e-1) {
    stellen = 10;
    offset = 0.1;
  } else if(zeit+offset >= 9.9e-2) {
    stellen = 9;
    offset = 0.01;
  } else offset = 0;
  if(stellen < F1_MIN_STELLEN) stellen = F1_MIN_STELLEN;
  return(stellen);
}

// bedingte Anzeige von Messwerten
void messwert_lcd1(char *s)
{
  if(!ausgabe_sperre) lcd_messwert_zeile1(s);
}  

void messwert_lcd2(char *s)
{
  if(!ausgabe_sperre) lcd_messwert_zeile2(s);
} 

// Routine zur Wandlung und Ausgabe eines Messwertes
void wandel_Fx(char *header, double x, char zeige_periode, char *s, uint8_t stellen)    // Anzeige von F, P oder UPM
{
int i,j,dez_punkt,dimension;
char *p;
  if(zeige_periode == WANDEL_F && nur_MHz)
  {
    if(x >= 1e6) stellen--;
    if(x >= 1e7) stellen--;
    if(x >= 1e8) stellen--;
    if(x >= 1e9) stellen--;
    sprintf(s,"%.*f MHz",stellen,x*1e-6);   // festes Format bei "nur MHz"
    if(*s == '0') *s = ' ';
    return;
  }    

  dez_punkt = 0;
  dimension = 3;
  p = header;
  while(*p) *s++ = *p++;                    // Bezeichner kopieren, wenn angegeben
  if(x >= 0.001) {                          // 1mHz ist Untergrenze
    if(zeige_periode == WANDEL_P) {
      x = 1/x;                              // Kehrwert bilden
      dimension=5;                          // und in Sekunden als Dimension
    }
    while(x<1.0) {x*=1000.0;dimension++;}   // in den Hz-Bereich bringen
    while(x>=1000.0) {x*=0.001;dimension--;}
    while(x >= 10.0) {
      x *= 0.1;
      dez_punkt++;
    }
    x += rundung[stellen];                  // runden
    if(x >= 10.0) {			                    // Ueberlauf bei Rundung
      x *= 0.1;                             // korrigieren
      dez_punkt++;
      if(dez_punkt > 2) {		                // Ueberlauf der Dimension
        dimension--;
        dez_punkt = 0;
      }
    }
  } else {
    x = 0.0; dez_punkt=0; dimension=3;      // 0.00000 Hz ausgeben
  }
  
  for(i=0;i<stellen;i++) {
    j = (uint8_t)x;
    *s++ = (j+'0');
    if(i == dez_punkt) {
      if(anzeige_format & 2)
        *s++ = (',');
      else
        *s++ = ('.');
    }  
    x -= j;
    x *= 10;
  }

  if(zeige_periode == WANDEL_D)
    p = u_dim[anzeige_format&1][dimension];
  else
    p = f_dim[anzeige_format&1][dimension];
  while(*p) *s++ = *p++;                    // Dimension anhaengen
  *s++ = 0;                                 // string abschliessen
  *s = 0;                                   // string abschliessen
  s[MAX_ERGEBNIS_STR] = 0;                  // zur Sicherheit begrenzen
}

void Fx_str_mittig(char *header, double x, char zeige_periode, char *s, uint8_t stellen)    // Anzeige von F, P oder UPM
{
char p[TEMP_STR_LEN];
int i,j;
  *p = 0;
  for(i = 0; i < MAX_ERGEBNIS_STR; i++)     // Ergebnisstring loeschen
    s[i] = ' ';
  s[MAX_ERGEBNIS_STR] = 0;                  // Ende von s
  
  wandel_Fx(header,x,zeige_periode,p,stellen);// wandeln
  i = strlen(p);
  i = lcd_breite - i;                       // auf 16 Zeichen/Zeile einmitten
  if(i < 0) i = 0;                          // Unterlauf verhindern
  j = i/2;
  i = 0;
  while(p[i]) s[j++] = p[i++];              // in den Ergebnisstring schreiben
  s[lcd_breite] = 0;                        // Laenge sicher begrenzen
} 



// steigende Flanken von CAP1 speichert PWM->CH2_CTR
void init_PIO0_SM0(void)
{
// PIO0 aktivieren  
  RESETS_CLR->RESET = RESETS_RESET_pio0_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_pio0_Msk));
// Programm zur Erkennung pos. Flanken  CAP1
// Eingangssignal CAP1 invertiert bewerten: geht schneller!  
// ADR0
  PIO0->INSTR_MEM0 = 0x00c0+2;      // jmp FIN1_PIN -> ADR2 wenn '1'  
// ADR1
  PIO0->INSTR_MEM1 = 0x0000+0;      // jmp ADR0 weiter auf '1' warten  
// ADR2
  PIO0->INSTR_MEM2 = 0x00c0+2;      // jmp FIN1_PIN -> ADR2 warten bis '0'
// input-pin 1 -> 0
// ADR3  
  PIO0->INSTR_MEM3 = 0x8000;        // push sofort nach CAP1 = aktiv
// ADR4  
  PIO0->INSTR_MEM4 = 0xc000;        // bei Bedarf set IRQ0
  
// fuer SM0: invertierter CAP1-Eingang
  IO_BANK0->GPIO17_CTRL = IO_BANK0_GPIO0_CTRL_INOVER_INVERT << IO_BANK0_GPIO0_CTRL_INOVER_Pos |
                          IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;
 
  PIO0->SM0_EXECCTRL = (3 << PIO0_SM0_EXECCTRL_WRAP_TOP_Pos) |        // autowrap fr max.
                       (0 << PIO0_SM0_EXECCTRL_WRAP_BOTTOM_Pos) |     // Geschwindigkeit
                       (CAP1_PIN  << PIO0_SM0_EXECCTRL_JMP_PIN_Pos);  // Pin fuer JMP
  
// Startadresse
  PIO0->SM0_INSTR =  0x0000;        // jmp ADR0
  PIO0->SM0_CLKDIV = 1 << PIO0_SM0_CLKDIV_INT_Pos |
                     0 << PIO0_SM0_CLKDIV_FRAC_Pos;
// starten  
  PIO0_SET->CTRL = (BIT(SM0)) << (PIO0_CTRL_SM_ENABLE_Pos);
}

// Zeitstempel erzeugen:
// DMA-Wert = Zeitpunkt, DMA-TRANS-COUNT = Ereignisse
void init_dma_pio0(void)
{
  RESETS_CLR->RESET = RESETS_RESET_dma_Msk | RESETS_RESET_busctrl_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_dma_Msk));
  BUSCTRL->BUS_PRIORITY = BUSCTRL_BUS_PRIORITY_DMA_W_Msk |
                          BUSCTRL_BUS_PRIORITY_DMA_R_Msk;

// PIO0_SM0  
  DMA->CH0_READ_ADDR = (uint32_t) &PWM->CH2_CTR;                // capture CH2-CTR
  DMA->CH0_WRITE_ADDR = (uint32_t) &PWM_CH2_event;              // und sichern
  DMA->CH0_TRANS_COUNT = 1;
  DMA->CH0_CTRL_TRIG =  DREQ_PIO0_RX0 << DMA_CH1_CTRL_TRIG_TREQ_SEL_Pos |     // PIO0_RXF0
                        1 << DMA_CH0_CTRL_TRIG_CHAIN_TO_Pos |   // trigger CH1
                        2 << DMA_CH0_CTRL_TRIG_DATA_SIZE_Pos |  // word
                        DMA_CH0_CTRL_TRIG_EN_Msk;               // enable
// triggered by CH0 and triggers CH0 again  
  DMA->CH1_READ_ADDR = (uint32_t) &PIO0->RXF0;                  // transfer FIFO loeschen     
  DMA->CH1_WRITE_ADDR = (uint32_t) &dma_dummy;                  // to dummy location
  DMA->CH1_TRANS_COUNT = 1;                                     // only 1 transfer
  DMA->CH1_CTRL_TRIG =  0x3f<<DMA_CH0_CTRL_TRIG_TREQ_SEL_Pos |  // free running
                        0 << DMA_CH0_CTRL_TRIG_CHAIN_TO_Pos |   // Trigger CH0  
                        2 << DMA_CH0_CTRL_TRIG_DATA_SIZE_Pos |  // word
                        DMA_CH0_CTRL_TRIG_EN_Msk;               // enable
}


// 1 kHz Aufrufe fuer Messzeiten F1_TIMER (PWM7)
// bzw. 50 kHz fuer Regressionsberechnung REG_TIMER (PWM6)
void PWM_IRQ_WRAP_IRQHandler(void)
{
static uint32_t F1_ms_alt;  // fuer grobe Messung in ms: Vorteilerumschaltung
uint32_t temp;

#ifndef CORE1_AKTIV
  if(PWM->INTS & BIT(REG_TIMER)) {          // PWM_CH6?
    PWM->INTR = BIT(REG_TIMER);             // clear flag
    if(!(SIO->GPIO_IN & BIT(INT_REQ))) {    // 0 = neuer Zeitstempel vorhanden
      F1_timeout_counter = 0;               // reset timeout

#ifdef INT128
      handle_F1_reg_64();                   // Verarbeiten der Zeitstempel
#else      
      handle_F1_reg();                      // Verarbeiten der Zeitstempel
#endif      
    }  
  }  
#endif
  
  if(PWM->INTS & BIT(F1_TIMER)) {           // 1 kHZ Aufrufe PWM_CH7?
    PWM->INTR = BIT(F1_TIMER);              // clear flag
    F1_mess_dauer++;                        // fuer Messablauf
    F1_timeout_counter++;                   // dto.
    temp = PWM->CH2_CTR;                    // neu ms-Wert Zaehlerstand lesen
// fuer maximal 65 MHz !
    F1_ms_grob = (temp - F1_ms_alt) & 0xffff;  // eff. Differenz
    F1_ms_alt = temp;                       // fuer naechsten Vergleich
    
// Umschaltung von Fref testen    
    if(SIO->GPIO_IN & BIT(REF_MUX_IN_OUT))  // Signal von Ladungspumpe
      temp = 1;                             // ext. 10 MHz Referenz
    else
      temp = 0;
    if(ex_ref_aktiv ^ temp) {               // geaenderte Referenz
      ex_ref_aktiv = temp;
      F1_neustart();                        // akt. Messung neu beginnen
    }  

    dekodiere_tasten();                     // einlesen und auswerten
// fertig-LED und 1pps-LED blinken lassen
    if(led_count) {                         // ready-LEDs
      SIO->GPIO_OUT_SET = BIT(LOCAL_LED) | BIT(EXT_LED);  // on
      led_count--;
    } else {
      SIO->GPIO_OUT_CLR = BIT(LOCAL_LED) | BIT(EXT_LED);  // off
    }  
  }
  __DSB();
}

// ms-Raster fuer Messablauf erzeugen
// 50 kHz per PWM6 fuer Regressionsberechnung starten
void init_F1_timer(void)
{
  RESETS_CLR->RESET = RESETS_RESET_pwm_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_pwm_Msk));
  IO_BANK0->GPIO15_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;  // LED-Ausgang
  IO_BANK0->GPIO25_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;  // lokal LED-Ausgang
  SIO->GPIO_OE_SET = BIT(LOCAL_LED) | BIT(EXT_LED); 
  
  PWM->CH7_DIV = 100 << PWM_CH0_DIV_INT_Pos;      // Vorteiler / 100
  PWM->CH7_TOP = SystemCoreClock/100/1000-1;      // 1 kHz
  PWM->CH7_CSR =  1;                              // enable ms-timer
  PWM_SET->INTE = BIT(F1_TIMER);                  // per iSR
  
#ifndef CORE1_AKTIV
  PWM->CH6_DIV = 100 << PWM_CH0_DIV_INT_Pos;      // Vorteiler / 100
  PWM->CH6_TOP = SystemCoreClock/100/REG_ABTASTRATE-1;     // 50 kHz
  PWM->CH6_CSR =  1;                              // enable reg-timer
  PWM_SET->INTE = BIT(REG_TIMER);                 // per iSR
#endif
  
  NVIC_EnableIRQ(PWM_IRQ_WRAP_IRQn);
  NVIC_SetPriority(PWM_IRQ_WRAP_IRQn,3);          // geringe Prioritaet reicht
  
// Fin mit PWM->CH2_CTR zaehlen  
  IO_BANK0->GPIO21_CTRL = IO_BANK0_GPIO21_CTRL_FUNCSEL_pwm_b_2;  // eingang PWM2B
  PWM->CH2_CSR =  PWM_CH2_CSR_DIVMODE_rise << PWM_CH0_CSR_DIVMODE_Pos | // +Flanke
                  PWM_CH0_CSR_EN_Msk;
}

void init_F1_io(void)
{
// INT_REQ GPIO14 bleibt auf Eingang
// CAP1 bleibt auf Eingang und wird bei PIO0_SM2 passend initialisiert
// FIN bleibt auf Eingang und wird bei PIO0_SM1 passend initialisiert
// EX_REF wird nicht verwendet
// GPIO22 wird nicht verwendet
  
// PRE_MUX_OUT fuer Vorteilerumschaltung 
  IO_BANK0->GPIO16_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;
  SIO->GPIO_OE_SET = BIT(PRE_MUX_OUT);
// GPIO18 Testausgang fuer Laufzeitmessung
  IO_BANK0->GPIO18_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;
  SIO->GPIO_OE_SET = BIT(GPIO18);
}  

  
void F1_neustart(void)
{
    F1_mess_dauer =  F1_messzeit;       // 1. Messung sofort abschliessen
    F1_messwert_vorhanden  = 0;         // und 1. Ergebnis verwerfen
    F1_timeout_counter = 0; 
    init_stat();                        // Statistik loeschen
    setze_F1_offset();
}    


void F1_messung(uint8_t init_timer)
{
static uint8_t  init = 0, alte_funktion = 255;
uint8_t eff_stellen, temp;
static char p_str[TEMP_STR_LEN];
static char s[TEMP_STR_LEN];

  if(init_timer) { 
    init = 0;                           // Anzeige + Messung initialisieren
    init_AS6501();
    init_F1_io();
    init_F1_timer();                    // 1 ms Takt und 50 kHz Regression
    init_dma_pio0();                    // time stamps DMA
    init_PIO0_SM0();                    // Capture fuer Zeitstempel
    setze_F1_offset();                  // beim Start festlegen
#ifdef CORE1_AKTIV
    starte_core1();
#endif      
  }
   
  if(!init) {                           // bei Neustart
    F1_mess_dauer =  F1_messzeit;       // 1. Messung sofort abschliessen
    F1_messwert_vorhanden  = 0;         // und 1. Ergebnis verwerfen
    F1_timeout_counter = 0; 
    init = 1;
    update_LCD = 1;
    init_stat();                        // Statistik loeschen
  }
 
  if(!ausgabe_sperre && (anzeige_funktion != alte_funktion || update_LCD)) {  
    update_LCD = 0;
    if(anzeige_funktion >= MAX_FUNKTION) 
      anzeige_funktion = F_UND_P;       // sinnvollen Wert einstellen
    alte_funktion = anzeige_funktion;
    loesche_lcd_zeile12();
    switch(anzeige_funktion) {
      case F_UND_P:
        lcd_zeile1("   Signal F?    ");
        lcd_zeile2("   Signal P?    ");
        break;
      case FREQUENZ:
        lcd_zeile1("    Frequenz    ");
        lcd_zeile2("    Signal?     ");
        break;
      case PERIODE:
        lcd_zeile1("    Periode     ");
        lcd_zeile2("    Signal?     ");
        break;
      case DREHZAHL:
        lcd_zeile1("    Drehzahl    ");
        lcd_zeile2("    Signal?     ");
        break;
      case FMEAN_UND_N:
        lcd_zeile1("  Fmean: n=     ");
        lcd_zeile2("    Signal?     ");
        break;
      case FMAX_FMIN:
        lcd_zeile1("   Fmax-Fmin    ");
        lcd_zeile2("    Signal?     ");
        break;
      case SDEV_UND_N:
        lcd_zeile1("  S-dev: n =    ");
        lcd_zeile2("    Signal?     ");
        break;
    }
  }
  
  if(F1_timeout_counter >= F1_timeout) {      // bei fehlenden Eingangsimpulsen
    init = 0;                                 // neu initialisieren
  }  
// grobe bewertung von Fin zur Auswahl des Vorteilers

  if(!auto_vorteiler_aktiv && F1_ms_grob > F1V_FMAX) 
    auto_vorteiler_aktiv = 1;                 // Vorteiler aktiv fuer f > 1 MHz
  if(auto_vorteiler_aktiv && F1_ms_grob < F1V_FMIN) 
    auto_vorteiler_aktiv = 0;                 // Vorteiler abschalten f < 400 kHz 
    
  if(F1_mess_dauer >= F1_messzeit) {          // wenn Mindestzeit erreicht
#ifdef INT128
    F1_ergebnis = bewerte_F1_messung_64();    // F1_ergebnis und eff_zeit1 berechnen
#else      
    F1_ergebnis = bewerte_F1_messung();       // F1_ergebnis und eff_zeit1 berechnen
#endif      
    if(F1_ergebnis > 0.0) {
// lokale Taktfrequenz wird mit Poti korrigiert
      if(!ex_ref_aktiv) F1_ergebnis *= f_ref_korrektur;
// digitalen Abgleich mit F1_offset durchfuehren   
      F1_ergebnis += F1_ergebnis*(F1_offset*F_OFFSET_SCHRITT);
      F1_mess_dauer = 0;                      // neue Messung beginnen
      if(F1_messwert_vorhanden) { 
        eff_stellen = F1_stellen;
        if(!eff_stellen) eff_stellen = berechne_stellen(eff_zeit1, reg_ereignisse);
        if(F1_vorteiler_aktiv)
          F1_ergebnis *= (double)F1_faktor;   // und ggf. skalieren
        stat_new(F1_ergebnis, eff_stellen);  
        led_count = led_ein_zeit;
// serielle Datenausgabe zuerst        
        if(ser_ausgabe_kanal == P1) {
          wandel_Fx("",F1_ergebnis, WANDEL_P, s, eff_stellen);
          sende_str1(s);
        }
        if(ser_ausgabe_kanal == UPM1) {
          wandel_Fx("",F1_ergebnis*60.0/F1_upm_divisor, WANDEL_D, s, eff_stellen);
          sende_str1(s);
        }
        wandel_Fx("",F1_ergebnis, WANDEL_F, s, eff_stellen);
        if(ser_ausgabe_kanal == F1) 
          sende_str1(s);
// danach Anzeige auf LCD        
        switch(anzeige_funktion) {
          case F_UND_P:
            Fx_str_mittig("F ",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            messwert_lcd1(p_str);
            Fx_str_mittig("P ",F1_ergebnis, WANDEL_P, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case FREQUENZ:
            Fx_str_mittig("",F1_ergebnis, WANDEL_F, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case PERIODE:
            Fx_str_mittig("",F1_ergebnis, WANDEL_P, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case DREHZAHL:
            Fx_str_mittig("",F1_ergebnis*60.0/F1_upm_divisor, WANDEL_D, s, eff_stellen);
            messwert_lcd2(s);
            break;
          case FMEAN_UND_N:
            strcpy(s,"Fmean: n=     ");
            long2a(stat_n, &s[10], 0);
            LCD_str_einmitten(s, lcd_breite);
            messwert_lcd1(s);
            Fx_str_mittig("\001 ",stat_mean, WANDEL_F, s, eff_stellen);
            messwert_lcd2(s);
            break;
          case FMAX_FMIN:
            Fx_str_mittig("\002 ",stat_max, WANDEL_F, p_str, eff_stellen);
            messwert_lcd1(p_str);
            Fx_str_mittig("\003 ",stat_min, WANDEL_F, p_str, eff_stellen);
            messwert_lcd2(p_str);
            break;
          case SDEV_UND_N:
            strcpy(s,"S-dev: n=     ");
            long2a(stat_n, &s[10], 0);
            LCD_str_einmitten(s, lcd_breite);
            messwert_lcd1(s);
            temp = anzeige_format;
            anzeige_format |= 1;
            if(eff_stellen > 10) eff_stellen = 10;  // bei 2x16 LCD noch darstellbar
            if(stat_n >= 1) 
              Fx_str_mittig("\345 ",stat_sdev(), WANDEL_F, s, eff_stellen);
            anzeige_format = temp;
            messwert_lcd2(s);
            break;
        }
      } else F1_messwert_vorhanden  = 1;    // sperre wieder aufheben
    }
  }   
}  


#ifdef CORE1_AKTIV

void core1_loop()
{
volatile int n;
  while(1) {
    switch(core1_cmd) {
    case CORE1_STOP:
      core1_cmd = CORE1_ACK;
    case CORE1_ACK:
      while(core1_cmd == CORE1_ACK);
      break;
    case CORE1_RUN:
      if(!(SIO->GPIO_IN & BIT(INT_REQ))) {    // 0 = neuer Zeitstempel vorhanden
        F1_timeout_counter = 0;               // reset timeout
#ifdef INT128
        handle_F1_reg_64();                      // Verarbeiten der Zeitstempel
#else      
        handle_F1_reg();                      // Verarbeiten der Zeitstempel
#endif      
      }  
      break;
    }  
  }
} 

void starte_core1(void) {
uint32_t cmd_tab[] = {0, 0, 1,                      // default
                     (uint32_t) 0x10000100,         // std. vec_tab 
                     (uint32_t)&core1_stack[STACK_LEN-1], // stack
                     (uint32_t)core1_loop};         // start
uint32_t seq = 0, cmd;
// reset core1
  PSM_SET->FRCE_OFF = PSM_FRCE_OFF_proc1_Msk;       // core1 passiv
  while(!(PSM->FRCE_OFF & PSM_FRCE_OFF_proc1_Msk)); // abwarten
  PSM_CLR->FRCE_OFF = PSM_FRCE_OFF_proc1_Msk;       // Bit wieder loeschen

  do {
    cmd = cmd_tab[seq];
    if (!cmd) {
      while ((SIO->FIFO_ST & SIO_FIFO_ST_VLD_Msk))  
        SIO->FIFO_RD;                               // FIFO loeschen
    }
    while (!(SIO->FIFO_ST & SIO_FIFO_ST_RDY_Msk));  // FIFO abwarten
    SIO->FIFO_WR = cmd;                             // neuer Befehl
    __SEV();                                        // trigger
    while (!(SIO->FIFO_ST & SIO_FIFO_ST_VLD_Msk))
      __WFE();
    if(cmd == SIO->FIFO_RD) seq++;                  // Antwort vergleichen
    else seq = 0;                   // bei Fehler: neuer Zyklus
  } while (seq < 6);                // Tabellenlaenge abarbeiten
}

#endif