/*
2023-04-30
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#define PICO_FM_AS65
#include "System-Pico-FM-AS65.h"

#define VERSION_LCD1 "mino PicoFM-AS65"
#define VERSION_LCD2 "V1.2  2023-11-08"
#define VERSION_SER  "mino Pico-FM-AS65 V1.2 2023-1-08"

char version[] = VERSION_SER;

uint32_t SystemCoreClock  = (DEF_SYSCLOCK); // 133 MHz
uint32_t F_xtal = MHZ_12;                   // onboard default

double f_ref_korrektur = 1.0;               // TCXO-Korrektur mit Poti
uint32_t ms_counter;                        // fuer freie Verwedung
volatile uint8_t ms_flag, adc_flag;         // ms-flags
uint8_t ex_ref_aktiv;


void kurz_warten(void)    // fuer kurze Verzoegerungen
{
  volatile int i = 1000;
  while(i--);
}  

// XOSC aktivieren, SystemCoreClock setzen und Taktsignale aktivieren
void init_clock(void)
{
volatile int i;
  VREG_AND_CHIP_RESET_SET-> VREG = 15<<VREG_AND_CHIP_RESET_VREG_VSEL_Pos;
  while(!(VREG_AND_CHIP_RESET->VREG & VREG_AND_CHIP_RESET_VREG_ROK_Msk));
  kurz_warten();       // Vreg abwarten

  XOSC->CTRL     = (XOSC_CTRL_FREQ_RANGE_1_15MHZ << XOSC_CTRL_FREQ_RANGE_Pos);
  XOSC->STARTUP  = 100;             // ca. 2 ms bei 12 MHz
  XOSC_SET->CTRL = (XOSC_CTRL_ENABLE_ENABLE << XOSC_CTRL_ENABLE_Pos);
  while (!(XOSC->STATUS & XOSC_STATUS_STABLE_Msk));
  
  RESETS_CLR->RESET = RESETS_RESET_pll_sys_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_pll_sys_Msk));


  PLL_SYS->CS = (REF_DIV << PLL_SYS_CS_REFDIV_Pos); // PLL-Aufloesung vorgeben
  PLL_SYS->FBDIV_INT = (SystemCoreClock*PLL_DIV2*REF_DIV)/(F_xtal/PLL_DIV1);  // PLL-VCO typ. 1310 MHz
// eff. Taktfrequenz ermitteln  
  PLL_SYS->PRIM = (PLL_DIV1 << PLL_SYS_PRIM_POSTDIV1_Pos) | 
                  (PLL_DIV2 << PLL_SYS_PRIM_POSTDIV2_Pos);
  SystemCoreClock = PLL_SYS->FBDIV_INT *(F_xtal/REF_DIV) / 
                    (PLL_DIV1*PLL_DIV2);

  PLL_SYS_CLR->PWR = PLL_SYS_PWR_VCOPD_Msk | PLL_SYS_PWR_PD_Msk;
  while (!(PLL_SYS->CS & PLL_SYS_CS_LOCK_Msk));

  PLL_SYS_CLR->PWR = PLL_SYS_PWR_POSTDIVPD_Msk;

  CLOCKS->CLK_REF_CTRL = (CLOCKS_CLK_REF_CTRL_SRC_xosc_clksrc << CLOCKS_CLK_REF_CTRL_SRC_Pos);
  CLOCKS->CLK_SYS_CTRL = (CLOCKS_CLK_SYS_CTRL_AUXSRC_clksrc_pll_sys << CLOCKS_CLK_SYS_CTRL_AUXSRC_Pos) |
                         (CLOCKS_CLK_SYS_CTRL_SRC_clksrc_clk_sys_aux << CLOCKS_CLK_SYS_CTRL_SRC_Pos);

  CLOCKS->CLK_PERI_CTRL = CLOCKS_CLK_PERI_CTRL_ENABLE_Msk |
      (CLOCKS_CLK_PERI_CTRL_AUXSRC_clk_sys << CLOCKS_CLK_PERI_CTRL_AUXSRC_Pos);

  CLOCKS->CLK_ADC_CTRL = CLOCKS_CLK_ADC_CTRL_ENABLE_Msk |
      (CLOCKS_CLK_ADC_CTRL_AUXSRC_xosc_clksrc << CLOCKS_CLK_ADC_CTRL_AUXSRC_Pos);

// enable GPIO
  RESETS_CLR->RESET = RESETS_RESET_io_bank0_Msk | 
                      RESETS_RESET_pads_bank0_Msk;
  while((RESETS->RESET_DONE & (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk)) !=
        (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk));
}

// POSTDIV2 / 2 => SystemCoreClock *= 2
void set_turbo_mode()
{
  PLL_SYS->PRIM = (PLL_DIV1 << PLL_SYS_PRIM_POSTDIV1_Pos) | 
                  (PLL_DIV2/2 << PLL_SYS_PRIM_POSTDIV2_Pos);
  SystemCoreClock *= 2;
  while (!(PLL_SYS->CS & PLL_SYS_CS_LOCK_Msk));
} 


// LCD_RS und LCD_RW auf 3k3 Widerstand testen und ggf. SystemCoreClock verdoppeln
int teste_turbo(void)
{
  RESETS_CLR->RESET = RESETS_RESET_io_bank0_Msk | 
                      RESETS_RESET_pads_bank0_Msk;
  while((RESETS->RESET_DONE & (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk)) !=
        (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk));
  IO_BANK0->GPIO1_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;
  IO_BANK0->GPIO2_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;
  PADS_BANK0_SET->GPIO1 = PADS_BANK0_GPIO1_PUE_Msk; // Pullup = 1
  PADS_BANK0_CLR->GPIO1 = PADS_BANK0_GPIO1_PDE_Msk; // Pulldown = 0
  PADS_BANK0_SET->GPIO2 = PADS_BANK0_GPIO2_PUE_Msk; // Pullup = 1
  PADS_BANK0_CLR->GPIO2 = PADS_BANK0_GPIO2_PDE_Msk; // Pulldown = 0
  
  SIO->GPIO_OE_SET =  BIT(1);
  SIO->GPIO_OUT_CLR = BIT(1);   // LCD_RS auf 0
  kurz_warten();
  if((SIO->GPIO_IN & BIT(2))) return(0); // LCD_RW ist nicht verbunden
  SIO->GPIO_OE_CLR =  BIT(1);   // LCD_RS passiv
  SIO->GPIO_OE_SET =  BIT(2);
  SIO->GPIO_OUT_CLR = BIT(1);   // LCD_RS auf 0
  kurz_warten();
  if((SIO->GPIO_IN & BIT(2))) return(0); // LCD_RW ist nicht verbunden
  return(1);    // turbo-modus angefordert
}  


// alle Millisekunden aufrufen
void TIMER_IRQ_0_IRQHandler(void)
{
  TIMER->INTR = TIMER_INTR_ALARM_0_Msk;
  TIMER->ALARM0 = TIMER->TIMELR + MS_TEILER;
  ms_counter++;
  ms_flag = 1;
  adc_flag = 1;
}

// 64 bit Timer mit 1 us Takt und loeschen
void init_timer(void)
{
  RESETS_CLR->RESET = RESETS_RESET_timer_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_timer_Msk));

  NVIC_EnableIRQ(TIMER_IRQ_0_IRQn);
  NVIC_SetPriority(TIMER_IRQ_0_IRQn,3);
  WATCHDOG->TICK = WATCHDOG_TICK_ENABLE_Msk | (F_xtal / MHZ_FAKTOR);
  TIMER->TIMELW = 0;
  TIMER->TIMEHW = 0;
  TIMER->ALARM0 = TIMER->TIMELR + MS_TEILER;
  TIMER->INTE = TIMER_INTE_ALARM_0_Msk;
}

void warten_ms(uint32_t ms)
{
  do {
    while(!ms_flag);
    ms_flag = 0;
  } while(--ms);
} 


void init_adc0(void)    // ADC-Kanal0 dauernd wandeln
{
  RESETS_CLR->RESET = RESETS_RESET_adc_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_adc_Msk));
  ADC->CS = ADC_CS_START_MANY_Msk | ADC_CS_EN_Msk |
            ADC_POTI << ADC_CS_AINSEL_Pos;
  PADS_BANK0->GPIO28 = PADS_BANK0_GPIO0_OD_Msk;   // output disable
}

uint32_t get_adc0(void) // aktuellen Wert lesen
{
  return(ADC->RESULT);
}  

// auf externen Trimmer testen und ggf. zur Frequenzkorrektur verwenden
void teste_abgleich(void)
{
static int32_t summe = ADC_MAX/2 * ADC_FILTER;  // startwert
static uint32_t init, poti_vorhanden;           // test R-trimm

  if(!init) {
    PADS_BANK0_SET->GPIO28 = PADS_BANK0_GPIO0_PUE_Msk;   // pullup = 1
    kurz_warten();
    if(get_adc0() <= 4000) poti_vorhanden = 1;
    PADS_BANK0_CLR->GPIO28 = PADS_BANK0_GPIO0_PUE_Msk;   // pullup = 0
    PADS_BANK0_SET->GPIO28 = PADS_BANK0_GPIO0_PDE_Msk;   // pulldown = 1
    kurz_warten();
    if(get_adc0() >= 100) poti_vorhanden = 1;
    PADS_BANK0_CLR->GPIO28 = PADS_BANK0_GPIO0_PDE_Msk;   // pulldown = 0
    init = 1;
  }  

  if(adc_flag && poti_vorhanden) {
    adc_flag = 0;
    summe -= summe/ADC_FILTER;
    summe += (get_adc0() - ADC_MAX/2);
// ADC-Korrekturwert von +/- 2047 auf +/- 2,047 ppm von AS6501_REF abbilden
    f_ref_korrektur = 1.0 + (double)summe/ADC_FILTER*1e-9;;
  }  
}  

void reset_io()
{
// reset diverse Peripherie beim programmstart
  RESETS_SET->RESET = RESETS_RESET_io_bank0_Msk | 
                      RESETS_RESET_pio0_Msk | 
                      RESETS_RESET_pio1_Msk | 
                      RESETS_RESET_dma_Msk | 
                      RESETS_RESET_pwm_Msk | 
                      RESETS_RESET_timer_Msk |                        
                      RESETS_RESET_pads_bank0_Msk;
  SIO->GPIO_OE_CLR = 0xffffffff;  // alle loeschen
  kurz_warten();
}

int main()
{
uint8_t ee_init, c;
int temp;
//  starte_core1();
  reset_io();  
  temp = teste_turbo();
  init_clock();

// gespeicherte Einstellungen laden; Default-Werte beim 1. Einschalten
  ee_read_uint8(EE_ADR_PARA_INIT, &ee_init);
  if(ee_init == EE_INIT_MUSTER) {                     // EE-Muster ist gueltig
      lese_ee_F1_parameter();                           // dann alte Einstellungen
      lese_ee_allgemein_parameter();
  } else {
      ee_write_uint8(EE_ADR_PARA_INIT, EE_INIT_MUSTER); // Kennung schreiben
      schreibe_ee_F1_parameter();                       // und danach Defaultwerte
      schreibe_ee_allgemein_parameter();
  }    

  lcd_init();
  lcd_zeile1(VERSION_LCD1);
  lcd_zeile2(VERSION_LCD2);
  if(temp) set_turbo_mode();  // SytemCoreClock *= 2
  init_timer();
  init_adc0();
  init_uart1();
  F1_messung(1);              // init
  while(ausgabe_sperre && TIMER->TIMELR <= 2000000) {
    F1_messung(0);            // 2 s abwarten
  }  
  ausgabe_sperre = 0;
  update_LCD = 1;             // anzeige freigeben

  while(1) {
    __WFE();
    F1_messung(0);            // run
    teste_abgleich();
    c = lese_taste();                       // F1  Messung wird auch hier aufgerufen
    if(c == PLUS_TASTE) {                   // angezeigte Werte direkt umschalten
      if(anzeige_funktion < MAX_FUNKTION-1) {
        anzeige_funktion++;
        update_LCD = 1;
        ee_write_uint8(EE_ADR_ANZEIGE_FKT, anzeige_funktion);
      }
    } else if(c == MINUS_TASTE) {           // angezeigte Werte direkt umschalten
      if(anzeige_funktion > 0) {
        anzeige_funktion--;
        update_LCD = 1;
        ee_write_uint8(EE_ADR_ANZEIGE_FKT, anzeige_funktion);
      }
    } else if(c == PLUS_MINUS) {            // angezeigte Werte direkt umschalten
      if(anzeige_funktion >= FMEAN_UND_N) {
        stat_n = 0;
        update_LCD = 1;
      }
    } else if(c == INIT_TASTE) {            // Bedienmenue aufrufen
      ausgabe_sperre = 1;                   // Messwertanzeige sperren
      FM_AS65_hauptmenue();
      ausgabe_sperre = 0;                   // wieder zulassen
      update_LCD = 1;                       // und Anzeige aktualisieren
    }  
  }  
}


